/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: geoifn.c,v 1.2 1995/11/03 22:26:51 deaven Exp $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "dmd/geometry.h"
#include "dmd/memory.h"
#include "dmd/tools.h"

char *usage = "usage: %s [options] [geometry]\n"
  "options:\n"
  "\t-n #       set number of points to plot\n"
  "\t-k #,#     set range of k to compute\n"
  ;

/* Compute sin(x) / x safely.
 */
static double sink(double x)
{
#define SINK_EPS 1.e-5
	if(x < SINK_EPS && x > -(SINK_EPS))
	  return (1.0 + (1.0 / 6.0) * x * x);
#undef SINK_EPS

	return sin(x) / x;
}

/* Read a geometry and write an interference function
 */
void main(int argc, char **argv)
{
	int i, j, ik, ntab, arg;
	Geometry *g;
	FILE *f;
	double k, dk, result;
	double *rtab = NULL;
	double kmin = 0.0;
	double kmax = -1.0;
	int npoint = 256;		/* # points to plot */
	char *fn = "-", *ofn = "-";	/* filenames */

	for(arg=1; arg<argc; ++arg) {
	  if(argv[arg][0] == '-') {
	    switch(argv[arg][1]) {
	      case 'n':
		sscanf(argv[++arg], "%d", &npoint);
		break;
	      case 'k':
		sscanf(argv[++arg], "%lf,%lf", &kmin, &kmax);
		break;
	      default:
		fprintf(stderr, usage, argv[0]);
		exit(1);
	    }
	  } else { /* geometry filename */
	    fn = argv[arg];
	  }
	}

	f = fopen_with_default(fn, "r", NULL);
	g = Gread(f);
	fclose(f);

	if(g == NULL) {
	  fprintf(stderr, "%s: cannot read geometry\n", argv[0]);
	  exit(1);
	}

	ntab = g->n * (g->n - 1) / 2;
	rtab = dmalloc(ntab);
	ntab = 0;
	for(i=0; i<(g->n); ++i)
	  for(j=i+1; j<g->n; ++j)
	    rtab[ntab++] = Gdist(g, i, j);

	if(kmax < 0.0) { /* has not been set */
	  double dmin = -1.0;
	  for(i=0; i<ntab; ++i)
	    if(rtab[i] < dmin || dmin < 0.0) dmin = rtab[i];
	  kmax = 3.0 * 2.0 * M_PI / dmin;
	}

	f = fopen_with_default(ofn, "w", NULL);
	fprintf(f, "2 #1=(k [bohr^-1]) #2=(I(k) %s)\n", g->name);
	dk = (kmax - kmin) / (npoint - 1);
	for(ik=0; ik<npoint; ++ik) {
	  k = kmin + dk * ik;
	  result = 0.0;
	  for(i=0; i<ntab; ++i)
	    result += sink(k * rtab[i]);
	  result *= (2.0 / (double) g->n);
	  fprintf(f, "%.10g %.10g\n", k, 1.0 + result);
	}
	fclose(f);
	myfree(rtab);
	Gfree(g);

	exit(0);
}
