/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: georotate.c,v 1.5 1996/01/31 20:09:15 deaven Exp $
 */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "dmd/geometry.h"
#include "dmd/tools.h"

char *usage = "usage: %s <options> [geometry]\n"
  "options:\n"
  "\t-o file     Specify output file\n"
  "\t-z a,b,c    Specify new z axis in lattice vector units\n"
  "\t-y a,b,c    Specify new y axis in lattice vector units\n"
  "\t-a x,y,z,theta\n"
  "\t            Specify axis to rotate around, and angle (degrees)\n"
  "\t-n num      Specify repeat counter for rotation\n"
  ;

static double *x_rot(double theta)
{
	static double r[3][3] = 
	  {{1.0, 0.0, 0.0},  {0.0, 1.0, 0.0},  {0.0, 0.0, 1.0}};
	r[1][2] = sin(theta);
	r[2][1] = -r[1][2];
	r[1][1] = cos(theta);
	r[2][2] = r[1][1];

	return ((double *) r);
}

static double *y_rot(double theta)
{
	static double r[3][3] = 
	  {{1.0, 0.0, 0.0},  {0.0, 1.0, 0.0},  {0.0, 0.0, 1.0}};
	r[0][2] = sin(theta);
	r[2][0] = -r[0][2];
	r[0][0] = cos(theta);
	r[2][2] = r[0][0];

	return ((double *) r);
}

static double *z_rot(double theta)
{
	static double r[3][3] = 
	  {{1.0, 0.0, 0.0},  {0.0, 1.0, 0.0},  {0.0, 0.0, 1.0}};
	r[0][1] = sin(theta);
	r[1][0] = -r[0][1];
	r[0][0] = cos(theta);
	r[1][1] = r[0][0];

	return ((double *) r);
}

static double *mul(double *a, double *b, double *r)
{
	int i, j, k;
	for(i=0; i<3; ++i)
	  for(j=0; j<3; ++j)
	  {
	    r[i*3 + j] = 0.0;
	    for(k=0; k<3; ++k)
	      r[i*3 + j] += a[i*3 + k] * b[k*3 + j];
	  }

	return r;
}

static Geometry *g_rotate(Geometry *g, double axis[3], double theta)
{
	double a[3*3], ai[3*3], r[3*3];
	double rxz, det;

	/* Create matrix to rotate z[] into axis[]...  */
	rxz = sqrt(axis[0] * axis[0] + axis[2] * axis[2]);
	mul(x_rot(-atan2(axis[1], rxz)),
	    y_rot(-atan2(axis[2], axis[0])), r);

	/* ...do the rotation... */
	mul(z_rot(theta), r, a);

	/* ...invert the first part... */
	matinv3(ai, r, &det);
	mul(ai, a, r);

	vmmul3(r, g->x, g->x, g->n);
	return g;
}

void main(int argc, char **argv)
{
	int i, j, arg;
	int set_zlv = 0, set_ylv = 0;
	int repeat = 1;
	int nin = 0, nout = 0;
	double axis[3] = {0.0, 0.0, 1.0};
	double theta = 0.0, basetheta = 0.0;
	double zlv[3], ylv[3];
	double x[3];
	double t[4*4], scam[3*3];
	Geometry *g;
	char *ifn = "-", *ofn = "-";
	FILE *fin, *fout;

	double z[3] = {0.0, 0.0, 1.0}, up[3] = {0.0, 1.0, 0.0};
	double origin[3] = {0.0, 0.0, 0.0};

	for(arg=1; arg<argc; ++arg)
	  if(argv[arg][0] == '-')
	  {
	    switch(argv[arg][1])
	    {
	      case 'a':
		sscanf(argv[++arg], "%lf,%lf,%lf:%lf",
		  axis, axis + 1, axis + 2, &theta);
		theta *= (M_PI / 180.0);
		break;

	      case 'n':
		sscanf(argv[++arg], "%d", &repeat);
		break;

	      case 'o':
		ofn = argv[++arg];
		break;

	      case 'z':
		sscanf(argv[++arg], "%lf,%lf,%lf",
		  zlv, zlv + 1, zlv + 2);
		set_zlv = 1;
		break;

	      case 'y':
		sscanf(argv[++arg], "%lf,%lf,%lf",
		  ylv, ylv + 1, ylv + 2);
		set_ylv = 1;
		break;

	      default:
	      	fprintf(stderr, usage, argv[0]);
	      	exit(1);
	    }
	  }
	  else
	    ifn = argv[arg];

	fin = fopen_with_default(ifn, "r", NULL);
	fout = fopen_with_default(ofn, "w", NULL);

	while((g = Gread(fin)) != NULL)
	{
	  ++nin; 

	  if(set_zlv || set_ylv) {
	    if(set_zlv) {
	      mmul3(g->box, zlv, z);
	    } else {
	      z[0] = z[1] = 0.0;
	      z[2] = 1.0;
	    }

	    if(set_ylv) {
	      mmul3(g->box, ylv, up);
	    } else {
	      up[0] = up[2] = 0.0;
	      up[1] = 1.0;
	    }

	    for(i=0; i<3; ++i) z[i] *= -1.0; /* to "view direction" */

	    Gentra(origin, z, up, t, scam);

	    for(i=0; i<g->n; ++i) {
	      Transf(g->x + 3*i, t, x);
	      for(j=0; j<3; ++j) (g->x + 3*i)[j] = x[j];
	    }
	  }

	  if(basetheta != 0.0) g_rotate(g, axis, basetheta);

	  for(i=0; i<repeat; ++i)
	  {
	    g_rotate(g, axis, theta);
	    Gprint(fout, g);
	    ++nout;
	    basetheta += theta;
	  }

	  Gfree(g);
	}

	if(fin != stdin) fclose(fin);
	if(fout != stdout) fclose(fout);

	fprintf(stderr, "%s: read %d, wrote %d geometries\n",
	  argv[0], nin, nout);

	exit(0);
}
