/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: geosed.c,v 1.6 1996/06/02 02:46:37 deaven Exp $
 */

#include <stdio.h>
#include <string.h>
#include "dmd/tools.h"
#include "dmd/geometry.h"
#include "dmd/element.h"
#include "dmd/memory.h"

char *usage = "usage: %s <options> [geometry]\n"
  "options:\n"
  "\t-b         Use box coordinate output (lattice)\n"
  "\t-f string  Reformat name using string template\n"
  "\t-g         Guess atomic identity instead of copying symbol\n"
  "\t-O         Set geometries to standard orientation\n"
  "\t-o fn      Write output to fn\n"
  "\t-p         Fix atoms to central box in periodic directions\n"
  "\t-P #       Apply GPrettyCell() with given rbond\n"
  "\t-s #       Output every # frames (default 1)\n"
  "\t-i #       Set `incomplete' flag\n"
  ;

/* Make a new title string based on the old one and a formatting
 * template (which must be writable).
 *
 * "%<letter><fmtopt>(keyword) --> printf("%<fmtopt><letter>, keyword-value)
 *
 * where <letter> can be d|e|f|g and the keyword-value is gleaned from
 * a keyword=value sequence in the title string.  If <letter> is %,
 * one percent sign is printed.
 */
static char *reformat(char *s, char *template)
{
	static char buf[128];
	char fmt, *t = template;

	buf[0] = '\0';

	while(*t != '\0') {
	  if(*t == '%') {
	    switch((fmt = *(t + 1))) {
	      case 'd':
	      case 'e':
	      case 'f':
	      case 'g':
		{
		  char *tok, *val, *fmtopt, *result;
		  char *r1, *r2; /* replaced chars in template */
		  char fmtbuf[64];
		  int idata;
		  double ddata;

		  if(*(fmtopt = t + 2) == '\0')
		    goto bad;
		  for(t = fmtopt + 1; *t != '('; ++t)
		    if(*t == '\0') goto bad;
		  *(r1 = t) = '\0'; /* replace '(' */

		  if(*(tok = t + 1) == '\0')
		    goto bad;
		  for(t = tok; *t != ')'; ++t)
		    if(*t == '\0') {*r1 = '('; goto bad; }
		  *(r2 = t) = '\0'; /* replace ')' */

		  if((val = strstr(s, tok)) == NULL)
		    result = "<not found>";
		  else {
		    while(*val != '=' && *val != '\0') ++val;
		    if(*val == '\0')
		      val = "0"; /* XXX */
		    else ++val;

		    sprintf(fmtbuf, "%%%s%c", fmtopt, fmt);
		    switch(fmt) {
		      case 'd':
			sscanf(val, "%d", &idata);
			sprintf(buf + strlen(buf), fmtbuf, idata);
			break;

		      case 'e':
		      case 'f':
		      case 'g':
			sscanf(val, "%lf", &ddata);
			sprintf(buf + strlen(buf), fmtbuf, ddata);
			break;

		      default:
			break;
		    }
		  }

		  *r1 = '(';
		  *r2 = ')';
		  t = r2 + 1;

		} /* end of case */
		break;

	      case '%':
		strcat(buf, "%");
		break;

	      default:
		goto bad;
		break;

	    } /* switch (fmt) */
	  } /* if (*t == %) */
	  else {
	    int len;
	    buf[(len = strlen(buf))] = *t;
	    buf[len + 1] = '\0';
	  }

	  ++t;

	}


	return buf;

bad:
	fprintf(stderr, "bad template `%s'\n", template);
	return s;
}

int main(int argc, char **argv)
{
	int i, m, arg;
	int guess = 0;
	int frame_mod = 1;
	int pflag = 0;
	int Pflag = 0;
	int bflag = 0;
	int Oflag = 0;
	int iflag = -1;
	double rbond = 1.0;
	char *template = NULL; /* no reformatting */
	int frames_in, frames_out;
	Geometry *g;
	Element *e;
	char *ifn = "-", *ofn = "-";
	FILE *fin, *fout;

	for(arg=1; arg<argc; ++arg)
	  if(argv[arg][0] == '-')
	    switch(argv[arg][1])
	    {
	      case 'o':
		ofn = argv[++arg];
		break;
	      case 'g':
		guess = 1;
		break;
	      case 's':
		sscanf(argv[++arg], "%d", &frame_mod);
		break;
	      case 'f':
		template = mystrdup(argv[++arg]);
		break;
	      case 'p':
		pflag = 1;
		break;
	      case 'P':
		Pflag = 1;
		sscanf(argv[++arg], "%lf", &rbond);
		break;
	      case 'b':
		bflag = 1;
		break;
	      case 'i':
		sscanf(argv[++arg], "%d", &iflag);
		iflag = (iflag > 0 ? 1 : 0);
		break;
	      case 'O':
		Oflag = 1;
		break;
	      default:
	      	fprintf(stderr, usage, argv[0]);
	      	exit(1);
	    }
	  else
	    ifn = argv[arg];

	fin = (strcmp(ifn, "-") == 0 ? stdin : fopen(ifn, "r"));
	if(fin == NULL)
	{
		fprintf(stderr, "%s: cannot open %s\n", argv[0], ifn);
		exit(1);
	}

	fout = (strcmp(ofn, "-") == 0 ? stdout : fopen(ofn, "w"));
	if(fout == NULL)
	{
		fprintf(stderr, "%s: cannot open %s\n", argv[0], ofn);
		exit(1);
	}

	frames_in = frames_out = 0;

	while((g = Gread(fin)) != NULL)
	{
	  ++frames_in;

	  if(pflag && (g->period & GP_PERIODIC))
	  {
	    for(i=0; i<(g->n); ++i) {
	      for(m=0; m<3; ++m) {
		if((g->period >> m) & 1) {
		  (g->x[i*3+m]) -= anint(g->x[i*3+m]);
		}
	      }
	    }
	  }

	  if(Pflag)
	    GPrettyCell(g, rbond);

	  if(guess) {
	    for(i=0; i<g->n; ++i)
	    {
	      e = GuessElement(g->aname[i]);
	      myfree(g->aname[i]);
	      g->aname[i] = e->name;
	    }
	  }

	  if(template != NULL) {
	    char *oldname = g->name;
	    g->name = reformat(oldname, template);
	    myfree(oldname);
	  }

	  if(Oflag)
	    Gstdorient(g);

	  if(iflag >= 0) {
	    if(iflag)
	      g->period |= GP_INCOMPLETE;
	    else
	      g->period &= ~GP_INCOMPLETE;
	  }

	  if(frames_in % frame_mod == 0) {
	    if(bflag)
	      Gprintbox(fout, g);
	    else
	      Gprint(fout, g);
	    ++frames_out;
	  }

	  if(guess) 
	    for(i=0; i<g->n; ++i) g->aname[i] = NULL;

	  if(template != NULL)
	    g->name = NULL;

	  Gfree(g);
	}
	if(fin != stdin) fclose(fin);
	if(fout != stdout) fclose(fout);

	fprintf(stderr, "%s: read %d, wrote %d geometr%s\n", argv[0],
	  frames_in, frames_out, 
	  frames_out > 1 ? "ies" : "y");

	return 0;
}
