/* GENETIC ALGORITHM AND RELATED CODE
 * Copyright (c) 1995 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: geosphere.c,v 1.2 1995/10/10 14:49:18 deaven Exp $
 */

/* Generate random coordinates.
 * Copyright (c) 1994 David Deaven
 */
#include <stdio.h>
#include <math.h>
#include "dmd/random.h"
#include "dmd/geometry.h"
#include "dmd/memory.h"

char *usage = "usage: %s <options>\n"
  "options:\n"
  "\t-n #     number of atoms\n"
  "\t-N #     number of molecules to generate\n"
  "\t-dmin #  smallest allowable internuclear distance\n"
  "\t-dmax #  largest allowable distance between groups of nuclei\n"
  "\t-a name  label of each nucleus (e.g. \"-a C\" for carbon)\n"
  "\t-o file  file to write to\n"
  "\t-r #     set radius explicitly\n"
  "\t-s #     random seed\n";

extern int ran3_seed;

#define PI M_PI

static void polar(double r, double theta, double phi, double x[3])
{
	x[0] = r * sin(theta) * cos(phi);
	x[1] = r * sin(theta) * sin(phi);
	x[2] = r * cos(theta);
}

static double cube[9] = {
  1.0, 0.0, 0.0,
  0.0, 1.0, 0.0,
  0.0, 0.0, 1.0
};

void main(int argc, char **argv)
{
  int arg, i, j;
  int n = 2;
  int N = 1;
  double dmin = 1.0;
  double dmax = 8.0;
  double radius = -1.0; 
  char *aname = NULL;
  char *ofn = "-";
  Geometry *g = NULL;
  FILE *f = NULL;

  for(arg=1; arg<argc; ++arg) {
    if(argv[arg][0] == '-') {
      switch(argv[arg][1]) {
	case 'n':
	  sscanf(argv[++arg], "%d", &n);
	  break;
	case 'N':
	  sscanf(argv[++arg], "%d", &N);
	  break;
	case 'r':
	  sscanf(argv[++arg], "%lf", &radius);
	  break;
	case 's':
	  sscanf(argv[++arg], "%d", &ran3_seed);
	  break;
	case 'a':
	  aname = argv[++arg];
	  break;
	case 'o':
	  ofn = argv[++arg];
	  break;
	case 'd':
	  if(strcmp(argv[arg], "-dmin") == 0)
		sscanf(argv[++arg], "%lf", &dmin);
	  else if(strcmp(argv[arg], "-dmax") == 0)
		sscanf(argv[++arg], "%lf", &dmax);
	  else
		fprintf(stderr, "%s: -d{min,max} only\n", argv[0]);
	  break;
	default:
	  fprintf(stderr, usage, argv[0]);
	  exit(1);
      }
    } else {
      fprintf(stderr, usage, argv[0]);
      exit(1);
    }
  }

  if(radius < 0.0)
    radius = sqrt((double) n / PI) * dmax;

  f = (strcmp(ofn, "-")==0 ? stdout : fopen(ofn, "w"));
  if(f == NULL)
  {
  	fprintf(stderr, "%s: cannot open %s\n", argv[0], ofn);
  	exit(1);
  }

  if(aname == NULL) aname = "C";
  g = Gmalloc(n, "random");
  for(i=0; i<n; ++i) g->aname[i] = aname;
  Gsetbox(g, cube);
  for(arg=0; arg<N; ++arg)
  {
    for(i=0; i<n; ++i)
      do {
	polar(radius, ran3() * PI, ran3() * 2.0 * PI, g->x + 3*i);
	for(j=0; j<i && Gdist(g, i, j) >= dmin; ++j);
      } while(j < i);
    Gprint(f, g);
  }
  if(f != stdout) fclose(f);
  for(i=0; i<n; ++i) g->aname[i] = NULL;
  Gfree(g);

  exit(0);
}
